-- ------------------------------------------------------------------------------------------------
-- TODO
-- ------------------------------------------------------------------------------------------------

-- mouse over gives skill reference (optional)
-- optional minimap icon:
-- 	left click = configuration
-- 	right click = toggle hidden UI elements
-- 	shift click = lock/unlock all group frames
-- validate shaman totems, DK stuff, paladin stances(seals), etc
-- ability to move default blizzard UI frames

-- ------------------------------------------------------------------------------------------------
-- Core functionality, the Pol_Monitor object.
-- ------------------------------------------------------------------------------------------------

-- Change layout for groups
-- (Re-order monitors within their groups)
-- 
-- Persistent storage of group/layout/monitor setup.

local function L(text) return text end  -- Localization stub
Pol_Monitor = LibStub("AceAddon-3.0"):NewAddon("PolMonitor", "AceConsole-3.0")

local options = {
	type="group",
	handler = Pol_Monitor,
	args = {
	
		frames = {
			type = "group",
			name = L("UI/Frame Options"),
			desc = L("Frame management options for hidding/showing/moving UI frames based on events"),
			args = {				
			
				fmain_display = {
					type = "select",
					style = "dropdown",
					name = L("Main HotBar Display"),
					desc = L("When should the default main hotbar be displayed?"),
					values = {always="Always", never="Never", any="Any Target or Combat", combat="In Combat", combo="Hostile Target or Combat", ooc="Out of Combat"},
					get = function(info) if not Pol_Monitor.db.profile.fmain_display then return "always" end return Pol_Monitor.db.profile.fmain_display end,
					set = function(info, v) Pol_Monitor.db.profile.fmain_display = v Pol_Monitor:UpdateFrameState() end,
					order = 1,
				},
									
				fblbar_display = {
					type = "select",
					style = "dropdown",
					name = L("Bottom Left HotBar Display"),
					desc = L("When should the bottom left hotbar be displayed?"),
					values = {always="Always", never="Never", any="Any Target or Combat", combat="In Combat", combo="Hostile Target or Combat", ooc="Out of Combat"},
					get = function(info) if not Pol_Monitor.db.profile.fblbar_display then return "always" end return Pol_Monitor.db.profile.fblbar_display end,
					set = function(info, v) Pol_Monitor.db.profile.fblbar_display = v Pol_Monitor:UpdateFrameState() end,
					order = 2,
				},

				fbrbar_display = {
					type = "select",
					style = "dropdown",
					name = L("Bottom Right HotBar Display"),
					desc = L("When should the bottom right hotbar be displayed?"),
					values = {always="Always", never="Never", any="Any Target or Combat", combat="In Combat", combo="Hostile Target or Combat", ooc="Out of Combat"},
					get = function(info) if not Pol_Monitor.db.profile.fbrbar_display then return "always" end return Pol_Monitor.db.profile.fbrbar_display end,
					set = function(info, v) Pol_Monitor.db.profile.fbrbar_display = v Pol_Monitor:UpdateFrameState() end,
					order = 3,
				},

				fr1bar_display = {
					type = "select",
					style = "dropdown",
					name = L("Right 1 HotBar Display"),
					desc = L("When should the right 1 hotbar be displayed?"),
					values = {always="Always", never="Never", any="Any Target or Combat", combat="In Combat", combo="Hostile Target or Combat", ooc="Out of Combat"},
					get = function(info) if not Pol_Monitor.db.profile.fr1bar_display then return "always" end return Pol_Monitor.db.profile.fr1bar_display end,
					set = function(info, v) Pol_Monitor.db.profile.fr1bar_display = v Pol_Monitor:UpdateFrameState() end,
					order = 4,
				},

				fr2bar_display = {
					type = "select",
					style = "dropdown",
					name = L("Right 2 HotBar Display"),
					desc = L("When should the right 2 hotbar be displayed?"),
					values = {always="Always", never="Never", any="Any Target or Combat", combat="In Combat", combo="Hostile Target or Combat", ooc="Out of Combat"},
					get = function(info) if not Pol_Monitor.db.profile.fr2bar_display then return "always" end return Pol_Monitor.db.profile.fr2bar_display end,
					set = function(info, v) Pol_Monitor.db.profile.fr2bar_display = v Pol_Monitor:UpdateFrameState() end,
					order = 5,
				},
				
				fplayer_display = {
					type = "select",
					style = "dropdown",
					name = L("Player Frame Display"),
					desc = L("When should the player frame be displayed?"),
					values = {always="Always", never="Never", any="Any Target or Combat", combat="In Combat", combo="Hostile Target or Combat", ooc="Out of Combat"},
					get = function(info) if not Pol_Monitor.db.profile.fplayer_display then return "always" end return Pol_Monitor.db.profile.fplayer_display end,
					set = function(info, v) Pol_Monitor.db.profile.fplayer_display = v Pol_Monitor:UpdateFrameState() end,
					order = 6,
				},
				
				fqtrack_display = {
					type = "select",
					style = "dropdown",
					name = L("Quest Tracker Display"),
					desc = L("When should the objective tracker frame be displayed?"),
					values = {always="Always", never="Never", any="Any Target or Combat", combat="In Combat", combo="Hostile Target or Combat", ooc="Out of Combat"},
					get = function(info) if not Pol_Monitor.db.profile.fqtrack_display then return "always" end return Pol_Monitor.db.profile.fqtrack_display end,
					set = function(info, v) Pol_Monitor.db.profile.fqtrack_display = v Pol_Monitor:UpdateFrameState() end,
					order = 7,
				},
				
				fmap_display = {
					type = "select",
					style = "dropdown",
					name = L("Mini Map Display"),
					desc = L("When should the mini map frame be displayed?"),
					values = {always="Always", never="Never", any="Any Target or Combat", combat="In Combat", combo="Hostile Target or Combat", ooc="Out of Combat"},
					get = function(info) if not Pol_Monitor.db.profile.fmap_display then return "always" end return Pol_Monitor.db.profile.fmap_display end,
					set = function(info, v) Pol_Monitor.db.profile.fmap_display = v Pol_Monitor:UpdateFrameState() end,
					order = 8,
				},								
			},
			order = 1
		},

		global = {
			type = "group",
			name = L("Global Options"),
			desc = L("Global settings for PolMonitor"),
			args = {
				alpha = {
					type = "range",
					name = L("Global Fade level"),
					desc = L("Sets the alpha value for all inactive monitors"),
					min = 0,
					max = 1,
					step = .05,
					get = function(info) return Pol_Monitor:GetAlpha() end,
					set = function(info, v) Pol_Monitor:SetAlpha(v) end,
					order = 0,
				},
				cooldownFade = {
					type = "toggle",
					name = L("Don't fade cooldowns"),
					desc = L("Note that this is a hack.  It may not work with all cooldown or skinning addons"),
					get = function(info) return not Pol_Monitor:GetCooldownFadeToggle() end,
					set = function(info, v) Pol_Monitor:SetCooldownFadeToggle(not v) end,
					order = 1,
				},
				buttonFacade = {
					type = "toggle",
					name = L("Register with Masque"),
					desc = L("If set, groups will be registered with Masque for skinning"),
					get = function(info) return Pol_Monitor:GetButtonFacadeToggle() end,
					set = function(info, v) Pol_Monitor:SetButtonFacadeToggle(v) end,
					hidden = not IsAddOnLoaded("Masque"),
					order = 2,
				},
				gcd = {
					type = "toggle",
					name = L("Ignore Global Cooldown"),
					desc = L(
						"If set, the global cooldown will not trigger cooldown monitors.\n\n"..
						"NOTE: For this to work you must enter the name of a spell that is only "..
						"affected by the global cooldown below (like Find Herbs)"),
					get = function(info) return Pol_Monitor:GetGlobalCooldownToggle() end,
					set = function(info, v) Pol_Monitor:SetGlobalCooldownToggle(v) end,
					order = 4,
				},
				gcdspell = {
					type = "input",
					name = L("Global Cooldown spell"),
					desc = L("Spell to check for the Global Cooldown"),
					get = function(info) return Pol_Monitor:GetGlobalCooldownSpell() end,
					set = function(info, v) Pol_Monitor:SetGlobalCooldownSpell(v) end,
					order = 5,
				},
				
			},
			order = 2,
		},
				
		group = {
			type = "group",
			name = L("Buttons/Monitors"),
			desc = L("Group management of button/monitors"),
			args = {
				add = {
					type = "input",
					name = L("New group"),
					desc = L("Add a new group"),
					get = false,
					set = "OnAddGroup",
					usage = L("<groupName>"),
					order = 1,
				},
			},
			order = 3,
		},
	},
}

local defaults = {
	profile = {groups={}},
	global = {spellTextures={}, itemTextures={}},
}


-- ------------------------------------------------------------------------------------------------
-- Event handlers
-- ------------------------------------------------------------------------------------------------

local inCombatState = false and (InCombatLockdown() == 1)
--local inCombatState = false

function Pol_Monitor:GetFrameState(dtype)

	--print("---")
	--if dtype then print ("debug dtype=" .. dtype) else print("debug dtype=nil") end
	--if inCombatState ~= nil then if inCombatState then print ("debug combat=true") else print ("debug combat=false") end else print("debug combat=nil") end
	--if UnitExists("target") then print ("debug exists=true") else print("debug exists=false") end

	if dtype == nil or dtype == "" or dtype == "always" then return true end

	if dtype == "any" and (UnitExists("target") or inCombatState) then return true end
	
	if dtype == "combo" and (UnitCanAttack("player", "target") or inCombatState) then return true end
	
	if (dtype == "combat" or dtype == "combo") and inCombatState then return true end
				
	if dtype == "ooc" and (not inCombatState) then return true end
	
	return false	
end

function Pol_Monitor:UpdateFrameState()

	Pol_Monitor:ToggleBlizzardUI (
		not (Pol_Monitor:GetFrameState(Pol_Monitor.db.profile.fplayer_display)),
		not (Pol_Monitor:GetFrameState(Pol_Monitor.db.profile.fmain_display)),
		not (Pol_Monitor:GetFrameState(Pol_Monitor.db.profile.fblbar_display)),
		not (Pol_Monitor:GetFrameState(Pol_Monitor.db.profile.fbrbar_display)),
		not (Pol_Monitor:GetFrameState(Pol_Monitor.db.profile.fr1bar_display)),
		not (Pol_Monitor:GetFrameState(Pol_Monitor.db.profile.fr2bar_display)),
		not (Pol_Monitor:GetFrameState(Pol_Monitor.db.profile.fqtrack_display)),
		not (Pol_Monitor:GetFrameState(Pol_Monitor.db.profile.fmap_display))
	)

end

function Pol_Monitor:OnInitialize()
    -- Rescue old ace2 cached icon paths if needed
	if Pol_MonitorDB and Pol_MonitorDB.account and not Pol_MonitorDB.global then
		Pol_MonitorDB.global = Pol_MonitorDB.account
		Pol_MonitorDB.account = nil
	end
	
	-- Back to normal
    self.db = LibStub("AceDB-3.0"):New("PolMonitorDB", defaults)
	self.groupObservers = {}
	self.groups = {}
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("PolMonitor", options)
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	
	-- Add dual-spec support
	local LibDualSpec = LibStub('LibDualSpec-1.0')
	LibDualSpec:EnhanceDatabase(self.db, "PolMonitor")
	LibDualSpec:EnhanceOptions(options.args.profile, self.db)

    self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("PolMonitor", "PolMonitor")
	self:RegisterChatCommand("polmonitor", "ChatCommand")
	self:RegisterChatCommand("pm", "ChatCommand")
	
	print ("|cff33ff99PolMonitor: |cffffffffType /pm or /polmonitor for configuration options!")
	
	-- We need to install this crazy event handler here, so we can handle Pet battle weirdness
	
	local function globalEvents(self, event, ...)
	
		if event == "PET_BATTLE_OPENING_START" then
		
			Pol_Monitor:ToggleBlizzardUI(true,true,true,true,true,true)
			
		elseif event == "ADDON_LOADED" then
		
			Pol_Monitor:UpdateFrameState()
			
		elseif event == "PLAYER_TARGET_CHANGED" then
		
			Pol_Monitor:UpdateFrameState()			
			
		elseif event == "PLAYER_REGEN_ENABLED" then
		
			inCombatState = false
			--print("player_regen_enabled() incombatstate set to false");
		
			Pol_Monitor:UpdateFrameState()

		elseif event == "PLAYER_REGEN_DISABLED" then
		
			inCombatState = true
			--print("player_regen_disabled() incombatstate set to true");
		
			Pol_Monitor:UpdateFrameState()
	
		end
	end
		
	local eventFrame = CreateFrame("FRAME", "PolMonitorFrame")
	
	eventFrame:RegisterEvent("ADDON_LOADED")	
	eventFrame:RegisterEvent("PLAYER_REGEN_DISABLED")
	eventFrame:RegisterEvent("PLAYER_REGEN_ENABLED")
	eventFrame:RegisterEvent("PLAYER_TARGET_CHANGED")
	eventFrame:RegisterEvent("PET_BATTLE_OPENING_START")
	eventFrame:RegisterEvent("PET_BATTLE_CLOSE")
	
	eventFrame:SetScript("OnEvent", globalEvents)		
end

function Pol_Monitor:ChatCommand(input)
    if not input or input:trim() == "" then
        LibStub("AceConfigDialog-3.0"):Open("PolMonitor")
    else
        LibStub("AceConfigCmd-3.0").HandleCommand(Pol_Monitor, "pm", "PolMonitor", input)
    end
end

function Pol_Monitor:OnEnable()
	self:OnProfileChanged()
end

function Pol_Monitor:OnProfileChanged()
	self.profileIsChanging = true
	-- Out with the old
	for groupName, group in pairs(self.groups) do
		self:_DeleteGroup(groupName)
	end
	-- In with the new
	for groupName, profile in pairs(self.db.profile.groups) do
		group = Pol_MonitorGroupFactory(groupName, profile)
		self:_AddGroup(groupName, group)
	end
	self.profileIsChanging = false
	
	
end

function Pol_Monitor:ToggleBlizzardUI(hidePlayerFrame, hideMainBar, hideBarBL, hideBarBR, hideBarL, hideBarR, hideQTrack, hideMap)

	local HiddenFrame = CreateFrame("Frame");
	HiddenFrame:Hide()
	
	if hidePlayerFrame then
		PlayerFrame:SetParent(HiddenFrame);
		PlayerFrame:Hide()
	else
		PlayerFrame:SetParent("UIParent");
		PlayerFrame:Show();
	end
	
	if hideMainBar then
	
		if strsub(GetBuildInfo(), 1, 1) == "5" then
			PVPMicroButton:Hide()
		end

		MainMenuExpBar:SetParent(HiddenFrame);
		MainMenuExpBar:Hide()

		MainMenuBarMaxLevelBar:SetParent(HiddenFrame);
		MainMenuBarMaxLevelBar:Hide()	
		
		MainMenuBarLeftEndCap:Hide()
		MainMenuBarRightEndCap:Hide()
		MainMenuBarTexture0:Hide();
		MainMenuBarTexture1:Hide();
		MainMenuBarTexture2:Hide();
		MainMenuBarTexture3:Hide();
		MainMenuBarPageNumber:Hide();
		ActionBarUpButton:Hide()
		ActionBarDownButton:Hide()
		MainMenuBarBackpackButton:Hide()
		CharacterBag0Slot:Hide()
		CharacterBag1Slot:Hide()
		CharacterBag2Slot:Hide()
		CharacterBag3Slot:Hide()
		CharacterMicroButton:Hide()
		SpellbookMicroButton:Hide()
		TalentMicroButton:Hide()
		--AchievementMicroButton:Hide()
		AchievementMicroButton:SetAlpha(0)		
		QuestLogMicroButton:Hide()
		GuildMicroButton:Hide()
		LFDMicroButton:Hide()
		CompanionsMicroButton:Hide()
		EJMicroButton:Hide()
		StoreMicroButton:SetParent(HiddenFrame)
		StoreMicroButton:Hide()
		MainMenuMicroButton:Hide()		
	
		for count = 1, 12 do		
			_G["ActionButton" .. count]:SetAttribute("statehidden", true)
			_G["ActionButton" .. count]:Hide()
			
		end		
	else

		if strsub(GetBuildInfo(), 1, 1) == "5" then
			PVPMicroButton:Show()
		end
	
		MainMenuExpBar:SetParent("UIParent")
		MainMenuExpBar:Show()

		MainMenuBarMaxLevelBar:SetParent(UIParent);
		MainMenuBarMaxLevelBar:Show()	
		MainMenuBarLeftEndCap:Show()
		MainMenuBarRightEndCap:Show()
		MainMenuBarTexture0:Show();
		MainMenuBarTexture1:Show();
		MainMenuBarTexture2:Show();
		MainMenuBarTexture3:Show();
		MainMenuBarPageNumber:Show();
		ActionBarUpButton:Show()
		ActionBarDownButton:Show()
		MainMenuBarBackpackButton:Show()
		CharacterBag0Slot:Show()
		CharacterBag1Slot:Show()
		CharacterBag2Slot:Show()
		CharacterBag3Slot:Show()
		CharacterMicroButton:Show()
		SpellbookMicroButton:Show()
		TalentMicroButton:Show()
		AchievementMicroButton:SetAlpha(1)
		AchievementMicroButton:Show()
		QuestLogMicroButton:Show()
		GuildMicroButton:Show()
		LFDMicroButton:Show()
		CompanionsMicroButton:Show()
		EJMicroButton:Show()
		StoreMicroButton:SetParent(UIParent)
		StoreMicroButton:Show()
		MainMenuMicroButton:Show()
	
		for count = 1, 12 do
			_G["ActionButton" .. count]:Show()
			_G["ActionButton" .. count]:SetAttribute("statehidden", false)		
		end
	end
	
	if hideBarBL then
		for count = 1, 12 do
			_G["MultiBarBottomLeftButton" .. count]:Hide()
			_G["MultiBarBottomLeftButton" .. count]:SetAttribute("statehidden", true)			
		end

	else
		for count = 1, 12 do
			_G["MultiBarBottomLeftButton" .. count]:Show()
			_G["MultiBarBottomLeftButton" .. count]:SetAttribute("statehidden", false)			
		end
	end

	if hideBarBR then
		for count = 1, 12 do
			_G["MultiBarBottomRightButton" .. count]:Hide()
			_G["MultiBarBottomRightButton" .. count]:SetAttribute("statehidden", true)					
		end

	else
		for count = 1, 12 do
			_G["MultiBarBottomRightButton" .. count]:Show()
			_G["MultiBarBottomRightButton" .. count]:SetAttribute("statehidden", false)			
		end
	end

	if hideBarL then
		for count = 1, 12 do
			_G["MultiBarLeftButton" .. count]:Hide()
			_G["MultiBarLeftButton" .. count]:SetAttribute("statehidden", true)						
		end

	else
		for count = 1, 12 do
			_G["MultiBarLeftButton" .. count]:Show()
			_G["MultiBarLeftButton" .. count]:SetAttribute("statehidden", false)						
		end
	end

	if hideBarR then
		for count = 1, 12 do
			_G["MultiBarRightButton" .. count]:Hide()
			_G["MultiBarRightButton" .. count]:SetAttribute("statehidden", true)					
		end

	else
		for count = 1, 12 do
			_G["MultiBarRightButton" .. count]:Show()
			_G["MultiBarRightButton" .. count]:SetAttribute("statehidden", false)			
			
		end
	end
	
	if hideMap then
		MinimapCluster:Hide()
	else
		MinimapCluster:Show()
	end
	
--	if hideChat then
--		FriendsMicroButton:Hide()
--		ChatFrameMenuButton:Hide()
--		
--		for count = 1, NUM_CHAT_WINDOWS do
--			if _G["ChatFrame" .. count]:IsVisible() then
--				_G["ChatFrame"..count]:Hide();
--				_G["ChatFrame"..count.."Tab"]:SetParent(HiddenFrame)
--				_G["ChatFrame"..count.."Tab"]:Hide();
--			end
--		end
--		
--	else
--		FriendsMicroButton:Show()
--		ChatFrameMenuButton:Show()
--		
--		for count = 1, NUM_CHAT_WINDOWS do
--			if _G["ChatFrame" .. count]:IsVisible() then	
--				_G["ChatFrame"..count]:Show();
--				_G["ChatFrame"..count.."Tab"]:SetParent(UIParent)
--				_G["ChatFrame"..count.."Tab"]:Show();
--			end
--		end		
--	end			
		
	if hideQTrack then
		if strsub(GetBuildInfo(), 1, 1) == "5" then
			WatchFrame:SetAlpha(0)
		else
			ObjectiveTrackerFrame:Hide()
		end
	else
		if strsub(GetBuildInfo(), 1, 1) == "5" then
			WatchFrame:SetAlpha(1)
		else
			ObjectiveTrackerFrame:Show()
		end
	end


end

-- ------------------------------------------------------------------------------------------------
-- Command handlers
-- ------------------------------------------------------------------------------------------------

--function Pol_Monitor:PrintAddonInfo()
--	self:Print(L("Monitors stuff."))
--end

function Pol_Monitor:OnAddGroup(info, groupName, ...)
	if self.groups[groupName] then
		self:Print(L("Group '%s' already exists"):format(groupName))
		return
	end
	
	group = Pol_MonitorGroupFactory(groupName, "Grid", ...)
	
	self.db.profile.groups[groupName] = group:GetProfileTable()
	
	self:_AddGroup(groupName, group)
end

function Pol_Monitor:OnDeleteGroup(groupName)
	self.db.profile.groups[groupName] = nil
	self:_DeleteGroup(groupName)
end

--function Pol_Monitor:OnListMonitorTypes()
--	for name, class in pairs(pol_monitorClasses) do
--		self:Print(" - "..name.." "..class:GetUsage().." - "..class:GetDescription())
--	end
--end

--function Pol_Monitor:OnListLayoutTypes()
--	for name, class in pairs(pol_layoutClasses) do
--		self:Print(" - "..name.." "..class:GetUsage().." - "..class:GetDescription())
--	end
--end

--function Pol_Monitor:OnListButtonTypes()
--	for name, class in pairs(pol_buttonClasses) do
--		self:Print(" - "..name.." "..class:GetUsage().." - "..class:GetDescription())
--	end
--end


-- ------------------------------------------------------------------------------------------------
-- Options handlers
-- ------------------------------------------------------------------------------------------------

function Pol_Monitor:GetAlpha()
	return self.db.profile.alpha or 0.2
end

function Pol_Monitor:SetAlpha(alpha)
	self.db.profile.alpha = alpha
	self:RepaintAllButtons()
end

function Pol_Monitor:GetCooldownFadeToggle()
	return self.db.profile.cooldownFadeToggle
end

function Pol_Monitor:SetCooldownFadeToggle(v)
	self.db.profile.cooldownFadeToggle = v
	self:RepaintAllButtons()
end

function Pol_Monitor:GetButtonFacadeToggle()
	return (self.db.profile.buttonFacadeToggle or false)
	-- or self.db.profile.buttonFacadeToggle == nil
end

function Pol_Monitor:SetButtonFacadeToggle(v)
	self.db.profile.buttonFacadeToggle = v
end

function Pol_Monitor:GetGlobalCooldownToggle()
	return self.db.profile.gcdToggle
end

function Pol_Monitor:SetGlobalCooldownToggle(v)
	self.db.profile.gcdToggle = v
end

function Pol_Monitor:GetGlobalCooldownSpell()
	return self.db.profile.gcdSpell
end

function Pol_Monitor:SetGlobalCooldownSpell(v)
	self.db.profile.gcdSpell = v
end

-- ------------------------------------------------------------------------------------------------
-- API
-- ------------------------------------------------------------------------------------------------

function Pol_Monitor:IgnoreAsGlobalCooldown(duration)
	if (self.gcdEndTime or 0) < GetTime() then
		local gcdstart, gcdduration = GetSpellCooldown(self.db.profile.gcdSpell or "")
		
		self.gcdEndTime = (gcdstart or 0) + (gcdduration or 0)
	end
	
	return self.db.profile.gcdToggle and (self.gcdEndTime > GetTime() and duration ~= gcdduration) or false
end

function Pol_Monitor:GetSpellTexture(spellName)

	if self.db.global.spellTextures[spellName] == nil then
		self.db.global.spellTextures[spellName] = GetSpellTexture(spellName)
	end
	
	return self.db.global.spellTextures[spellName] or "Interface\\Icons\\INV_Misc_QuestionMark"
end

function Pol_Monitor:SetSpellTexture(spellName, texture)

	if spellName and texture then
		self.db.global.spellTextures[spellName] = texture
	end
end

function Pol_Monitor:GetItemTexture(itemName)

	if self.db.global.itemTextures[itemName] == nil then
		local _, _, _, _, _, _, _, _, _, invTexture = GetItemInfo(itemName)
		self.db.global.itemTextures[itemName] = invTexture
	end
	
	return self.db.global.itemTextures[itemName] or "Interface\\Icons\\INV_Misc_QuestionMark"
end

function Pol_Monitor:GetGroups()

	local groups = {}
	
	for k, group in pairs(self.groups) do
		table.insert(groups, group)
	end
	
	return groups
end

function Pol_Monitor:RegisterGroupObserver(key, callback)
	self.groupObservers[key] = callback
end

function Pol_Monitor:UnRegisterGroupObserver(key)
	self.groupObservers[key] = nil
end

function Pol_Monitor:NotifyGroupObservers(action, group, monitor)
	if not self.profileIsChanging or not monitor then
		for key, callback in pairs(self.groupObservers) do
			callback(action, group, monitor)
		end
	end
end

function Pol_Monitor:RepaintAllButtons()
	for k, group in pairs(self.groups) do
		group:RepaintAllButtons()
	end
end


-- ------------------------------------------------------------------------------------------------
-- Internals
-- ------------------------------------------------------------------------------------------------

function Pol_Monitor:_AddGroup(groupName, group)

	self.groups[groupName] = group
		
	options.args.group.args[groupName] = group:GetOptionsTable(10)
	
	options.args.group.args[groupName].args.deleteSeparator = {
		type = "header",
		name = "",
		order = -2,
	}
	options.args.group.args[groupName].args.delete = {
		type = "execute",
		name = L("Delete"),
		desc = L("Delete this group"),
		func = function() self:OnDeleteGroup(groupName) end,
		order = -1,
	}
end

function Pol_Monitor:_DeleteGroup(groupName)

	self.groups[groupName]:delete()
	self.groups[groupName] = nil
	
	options.args.group.args[groupName] = nil
end

